package com.supwisdom.dlpay.framework.controller

import com.supwisdom.dlpay.framework.ResponseBodyBuilder
import com.supwisdom.dlpay.framework.core.JwtConfig
import com.supwisdom.dlpay.framework.core.JwtTokenUtil
import com.supwisdom.dlpay.framework.dao.ApiClientDao
import com.supwisdom.dlpay.framework.security.validate.ImageCodeUtil
import com.supwisdom.dlpay.framework.security.validate.VerifyCode
import com.supwisdom.dlpay.framework.domain.ApiClientRedis
import com.supwisdom.dlpay.framework.domain.TOperator
import com.supwisdom.dlpay.framework.redisrepo.ApiClientRepository
import com.supwisdom.dlpay.framework.security.OperUtil
import com.supwisdom.dlpay.framework.service.OperatorDetailService
import com.supwisdom.dlpay.framework.service.SystemUtilService
import com.supwisdom.dlpay.framework.util.HmacUtil
import com.supwisdom.dlpay.system.service.CommonService
import com.supwisdom.dlpay.system.service.FunctionService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.security.core.userdetails.UserDetails
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler
import org.springframework.social.connect.web.HttpSessionSessionStrategy
import org.springframework.stereotype.Controller
import org.springframework.ui.Model
import org.springframework.web.bind.annotation.*
import org.springframework.web.context.request.ServletWebRequest
import java.util.*
import javax.imageio.ImageIO
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse

@RestController
@RequestMapping("/api/auth")
class ApiAuthController {

    @Autowired
    lateinit var repo: ApiClientRepository

    @Autowired
    lateinit var apiClient: ApiClientDao

    @Autowired
    lateinit var systemUtil: SystemUtilService

    @Autowired
    lateinit var jwtConfig: JwtConfig

    @GetMapping("/gettoken")
    fun loginInit(appid: String): ResponseEntity<Any> {
        apiClient.findById(appid).run {
            if (!isPresent) {
                return ResponseEntity.status(HttpStatus.UNAUTHORIZED).build()
            }
            if (get().status != "normal") {
                return ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(1, "API状态错误"))
            }
            get()
        }.let {
            val token = generateRandomToken()
            val now = systemUtil.sysdatetime.hostdatetime
            ApiClientRedis().apply {
                id = appid
                loginTimestamp = now
                roles = it.roles
                this.token = HmacUtil.HMACSHA256(token, it.secret)
            }.also {
                repo.save(it)
            }
            return ResponseEntity.ok(ResponseBodyBuilder.create()
                    .data("token", token)
                    .data("timestamp", now)
                    .success())
        }
    }

    private fun generateRandomToken(): String {
        val random = ByteArray(12) { 0x00 }
        Random(System.currentTimeMillis()).nextBytes(random)
        return Base64.getEncoder().encode(random).toString(Charsets.UTF_8)
    }

    private fun checkSecretToken(api: ApiClientRedis, secret: String): Boolean {
        return (api.token == secret)
    }

    @GetMapping("/authentication")
    fun login(appid: String, secret: String): ResponseEntity<Any> {
        return repo.findById(appid).let {
            if (it.isPresent && checkSecretToken(it.get(), secret)) {
                it.get().also { client ->
                    client.token = ""
                    repo.save(client)
                }
                val token = JwtTokenUtil(jwtConfig).generateToken(
                        mapOf("uid" to appid, "issuer" to "supwisdom",
                                "subject" to "payapi",
                                "authorities" to it.get().roles.split(";")))
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .data("jwt", token)
                        .data("appid", appid)
                        .success())
            } else {
                ResponseEntity.status(HttpStatus.UNAUTHORIZED).build()
            }
        }
    }
}

@RestController
class ValidateCodeController {

    @GetMapping("/code/image")
    fun createCode(request: HttpServletRequest, response: HttpServletResponse) {
        val imageCode = VerifyCode(60)
        HttpSessionSessionStrategy().setAttribute(ServletWebRequest(request), ImageCodeUtil.LOGIN_IMAGECODE_SESSIONKEY, imageCode)
        ImageIO.write(imageCode.image, "JPEG", response.outputStream)
    }
}


data class LoginForm(val username: String, val password: String, val imageCode: String)

@Controller
class WebHomeController {
    @Autowired
    lateinit var functionService: FunctionService
    @Autowired
    lateinit var commonService: CommonService

    @GetMapping("/login")
    fun loginView() = "login"

    @GetMapping("/logout")
    fun logout(request: HttpServletRequest, response: HttpServletResponse): String {
        SecurityContextHolder.getContext().authentication?.also {
            SecurityContextLogoutHandler().logout(request, response, it)
        }
        return "redirect:/login?logout"
    }

    @GetMapping("/index")
    fun homeView(@AuthenticationPrincipal operUser: UserDetails, model: Model): String {
        model.addAttribute("loginOper", operUser as TOperator)
        val funclist = functionService.getFunctionsByOperid(operUser.operid)
        model.addAttribute("menus", functionService.getMenuTree(funclist, "-1"))
        model.addAttribute("payapiVersion", commonService.getSystemVersion())
        return "index"
    }
}
